@@ -225,26 +225,56 @@ def get_file(dir_name: str, muni: str = None, fileType: str = None):
     return file
 
 # function to download from a link
-def get_gdf_from_zipped_link(url, file_type:str, layer_name=None):
+def get_gdf_from_zipped_link(url, file_type:str, shapefile_name=None, layer_name=None, mask_gdf=None):
     """
+    mostly built for MassGIS direct downloads, bypassing the K drive.
 
     input: a URL that downloads a zipped shapefile, with file type (geojson, gdb, shp). if gdb, include layer name.
     output: a gdf of the download, with the shapefile download deleted (no storage need)
 
+    INPUTS:
+    - url:(string) URL for the download link (right click "download" button)
+    - file_type: (string) 'shp', 'gdb', 'geojson'. Do not include "."
+    
+    OPTIONAL PARAMETERS:
+    - shapefile_name: (string) for some MassGIS downloads, there are multiple shapefiles. Use this argument to 
+    specify the shapefile to read. Do not include .shp
+    - layer_name: (string) for GeoDatabase downloads, specifies which layer from the gdb to read in.
+    - mask_gdf: (gdf) a GeoDataFrame of a mask to read in 
     """
     path = download_and_extract_zip_to_temp(url)
 
     # pull out just the file type of choice
     
+    if shapefile_name is not None:
+        for dirpath, dirnames, filenames in os.walk(path):
+            for filename in filenames:
+                if filename.endswith('shp'):
+                    if shapefile_name in filename:
+                        file = os.path.join(dirpath, filename)
+        #assume it's a shapefile
+        if mask_gdf is not None:
+            gdf = gpd.read_file(file, mask=mask_gdf)
+        else:
+            gdf = gpd.read_file(file)
 
-    if file_type in ['shp', 'geojson']:
+    elif file_type in ['shp', 'geojson']:
         shapefile = get_file(dir_name=path, fileType= ("." + file_type))
         # read gdf from shapefile
-        gdf = gpd.read_file(shapefile)
+        if mask_gdf is not None:
+            gdf = gpd.read_file(shapefile, mask=mask_gdf)
+        else:
+            gdf = gpd.read_file(shapefile)
 
     elif file_type == 'gdb':
-        gdb = get_file(dir_name=path, fileType= ("." + file_type))
-        gdf = gpd.read_file(gdb, layer=layer_name)
+        for dirpath, dirnames, filenames in os.walk(path):
+            for dirname in dirnames:
+                if "gdb" in dirname:
+                    gdb = os.path.join(path, dirname)
+        if mask_gdf is not None:
+            gdf = gpd.read_file(gdb, layer=layer_name, mask=mask_gdf)
+        else:
+            gdf = gpd.read_file(gdb, layer=layer_name)
 
     gdf = gdf.to_crs(mass_mainland_crs)
 
@@ -253,6 +283,7 @@ def get_gdf_from_zipped_link(url, file_type:str, layer_name=None):
 
     return gdf
 
+
 def get_most_updated_state_assessors_data(muni, path=None, include_row=False):
     """
     pulls parcel data from the state's assessors website (with an exception for Boston who hosts separately)
@@ -659,7 +690,7 @@ def if_overlap (layer_1,
                 str).str.replace('[', '').str.replace(']', '').str.replace("'", '').str.lstrip(', ').str.rstrip(',').str.rstrip(', ')
 
     else:
-        overlap_grouped = overlap_only.groupby(by=id_field).reset_index()
+        overlap_grouped = overlap_only.groupby(by=id_field).agg('first').reset_index()
 
 
     #add a field for whether it overlaps or not. If inverse is not selected, a value of 1 indicates overlap. Reverse is true if inverse not selected.
